
//constants
LANGUAGES = ['en', 'jp'];

ERROR_OK	=	0;

ERROR_ITEM_NOT_FOUND = 404;

ERROR_FOLDER_NOT_EXISTED = 404;
ERROR_FOLDER_INVALID 	 = 405;
ERROR_PASSWORD_INVALID = 406;

DLNA_LICENSE_STATUS_LICENSED = 0;
DLNA_LICENSE_STATUS_NO_LICENSE = 1;
DLNA_LICENSE_STATUS_INVALID_LICENSE = 2;

DLNA_MEDIA_CLASS_CONTAINER = 1;
DLNA_MEDIA_CLASS_IMAGE = 2;
DLNA_MEDIA_CLASS_VIDEO = 3;
DLNA_MEDIA_CLASS_AUDIO = 4;
DLNA_MEDIA_CLASS_DOCUMENT = 5;

MOVE_ITEM_TYPE_DOWNLOAD			=	1;
MOVE_ITEM_TYPE_UPLOAD			=	2;

MOVE_ITEM_STATUS_WAITING		=	0;
MOVE_ITEM_STATUS_TRANSFERRING	=	1;
MOVE_ITEM_STATUS_PAUSED			=	2;
MOVE_ITEM_STATUS_COMPLETED		=	3;
MOVE_ITEM_STATUS_ABORTED_CANCEL	=	4;
MOVE_ITEM_STATUS_ABORTED_STORAGE_FULL	=	5;
MOVE_ITEM_STATUS_ABORTED_CONNECTION_BROKEN	=	6;

var ACTION_NONE = 0;
var ACTION_DELETE = 1;
var ACTION_MOVE = 2;
var ACTION_RENAME = 3;
var ACTION_CREATE_FOLDER = 4;

function MediaServer() {
	this.fm_fs_stat = function (path, on_fm_fs_stat)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/fm_fs_stat",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path': path}),					
					dataType: "json",
					success: function (data) {
						on_fm_fs_stat(data);
					}
				  });
		};
	this.fm_list_dir = function (path, only_folder, on_fm_list_dir)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/fm_list_dir",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path': path,'only_folder':only_folder}),					
					dataType: "json",
					success: function (data) {
						mediaitemlist_arib_word_convert(data.mediaitem_list);
						on_fm_list_dir(data.mediaitem_list);
					}
				  });
		};
	this.fm_move_file = function(path, destination, on_fm_move_file)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/fm_move_file",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path': path,'destination':destination}),
					dataType: "json",
					success: function (data) {
						on_fm_move_file(data.result);
					}
				  });
		}
	this.fm_rename_file = function(path, old_name, new_name, on_fm_rename_file)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/fm_rename_file",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path': path,'old_name':old_name,'new_name':new_name}),
					dataType: "json",
					success: function (data) {
						on_fm_rename_file(data.result);
					}
				  });
		}
	this.fm_delete_file = function(path, on_fm_delete_file)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/fm_delete_file",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path': path}),
					dataType: "json",
					success: function (data) {
						on_fm_delete_file(data.result);
					}
				  });
		}
	this.fm_create_folder = function(path, name, share_folder, on_fm_create_folder)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/fm_create_folder",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path': path, 'name': name, 'share_folder': share_folder}),
					dataType: "json",
					success: function (data) {
						on_fm_create_folder(data.result);
					}
				  });
		}
	this.get_parent_control = function (on_get_parent_control)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_parent_control",
					dataType: "json",
					success: function (data) {
						on_get_parent_control(data.has_password, data.age_limitation);
					}
				  });
		};
	this.set_parent_control = function(pass, age_limitation, on_set_parent_control)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/set_parent_control",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'password': pass,'age_limitation':age_limitation}),
					dataType: "json",
					success: function (data) {
						on_set_parent_control(data.result);
					}
				  });
		}
	this.change_password = function(old_pass, new_pass, on_change_password)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/change_password",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'old_password': old_pass,'new_password':new_pass}),
					dataType: "json",
					success: function (data) {
						on_change_password(data.result);
					}
				  });
		}
	this.verify_password = function(pass, on_verify_password)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/verify_password",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'password': pass}),
					dataType: "json",
					success: function (data) {
						on_verify_password(data.result);
					}
				  });
		}

	this.get_debug_level = function (on_get_debug_level)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_debug_level",
					dataType: "json",
					success: function (data) {
						on_get_debug_level(data.debug_level);
					}
				  });
		};
	this.set_debug_level = function(debug_level, on_set_debug_level)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/set_debug_level",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'debug_level': debug_level}),					
					dataType: "json",
					success: function (data) {
						on_set_debug_level(data);
					}
				  });
		}

	this.get_license_status = function (on_get_license_status)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_license_status",
					dataType: "json",
					success: function (data) {
						on_get_license_status(data);
					}
				  });
		};
	this.activate_license = function(license, on_activate_license)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/activate_license",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'license': license}),					
					dataType: "json",
					success: function (data) {
						on_activate_license(data);
					}
				  });
		}
	this.get_paired_device_list = function (on_get_paired_devices) 
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_paired_device_list",
					dataType: "json",
					success: function (data) {
						on_get_paired_devices(data.paired_device_list);
					}
				  });
		};

	this.remove_paired_device = function (device_id)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/remove_paired_device",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'device_id': device_id}),
					dataType: "json",
					success: function (data) {
					}
				  });
		};

    this.rescan_dms_device_list = function (on_rescan_dms_device_list)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/rescan_dms_device_list",
					dataType: "json",
					success: function (data) {
						on_rescan_dms_device_list(data);
					}
				  });
		};
		
	this.get_dms_device_list = function (on_get_dms_device_list)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_dms_device_list",
					dataType: "json",
					success: function (data) {
						on_get_dms_device_list(data.dms_device_list);
					}
				  });
		};

	this.remote_content_browse = function (udn, object_id, browse_children, start_index, request_count, on_remote_content_browse)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/remote_content_browse",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'udn':udn,'object_id':object_id, 'browse_children':browse_children,'start_index':start_index,'request_count':request_count}),
					dataType: "json",
					success: function (data) {
						mediaitemlist_arib_word_convert(data.mediaitem_list);
						on_remote_content_browse(data.mediaitem_list, data.total_matches);
					}
				  });
		};

	this.move_content_browse = function (path, start_index, request_count, in_progress, on_move_content_browse)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/move_content_browse",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path':path,'start_index':start_index,'request_count':request_count, 'in_progress':in_progress}),
					dataType: "json",
					success: function (data) {
						mediaitemlist_arib_word_convert(data.mediaitem_list);
						on_move_content_browse(data.mediaitem_list, data.total_matches);
					}
				  });
		};

	this.move_history_browse = function (start_index, request_count, on_move_history_browse)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/move_history_browse",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'start_index':start_index,'request_count':request_count}),
					dataType: "json",
					success: function (data) {
						mediaitemlist_arib_word_convert(data.mediaitem_list);
						on_move_history_browse(data.mediaitem_list, data.total_matches);
					}
				  });
		};
	
	this.delete_move_mediaitem = function (path, on_delete_move_mediaitem)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/delete_move_mediaitem",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path':path}),
					dataType: "json",
					success: function (data) {
						on_delete_move_mediaitem(data.result);
					}
				  });
		};

	this.delete_move_history = function (path, on_delete_move_history)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/delete_move_history",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path':path}),
					dataType: "json",
					success: function (data) {
						on_delete_move_history(data.result);
					}
				  });
		};
		
	this.start_move_mediaitem = function (udn, item_xml, sd_res_checked, on_start_move_mediaitem)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/start_move_mediaitem",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'udn':udn,'item_xml':item_xml,'sd_res_checked':sd_res_checked}),
					dataType: "json",
					success: function (data) {
						on_start_move_mediaitem(data.item);
					}
				  });
		};

	this.restart_move_mediaitem = function (path, on_start_move_mediaitem)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/start_move_mediaitem",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path':path}),
					dataType: "json",
					success: function (data) {
						on_start_move_mediaitem(data.item);
					}
				  });
		};
		
	this.cancel_move_mediaitem = function (path, on_cancel_move_mediaitem)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/cancel_move_mediaitem",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path':path}),
					dataType: "json",
					success: function (data) {
						on_cancel_move_mediaitem(data);
					}
				  });
		};

	this.get_move_status = function (path, on_get_move_status)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_move_mediaitem_status",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'path':path}),
					dataType: "json",
					success: function (data) {
						on_get_move_status(data.item);
					}
				  });
		};

	this.get_language = function (on_get_language)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_language",
					dataType: "json",
					success: function (data) {
						on_get_language(data.lang);
					}
				  });
		};

	this.set_language = function (lang, on_set_language)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/set_language",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'lang':lang}),
					dataType: "json",
					success: function (data) {
						on_set_language(data);
					}
				  });
		};

	this.get_dms_name = function (on_get_dms_name)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_dms_name",
					dataType: "json",
					success: function(data) {
						on_get_dms_name(data.name);
					}
				  });
		};

	this.set_dms_name = function (name)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/set_dms_name",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'name':name}),
					dataType: "json"
				  });
		};

	this.start_dms = function (on_start_dms)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/start_dms",
					dataType: "json",
					success: function(data) {
						on_start_dms(data);
					}
				  });
		};

	this.stop_dms = function ()
		{
			$.ajax({
					type: "post",
					url: "/api/v1/stop_dms",
					dataType: "json"
				  });
		};

	this.get_product_features = function (on_get_product_features)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_product_features",
					dataType: "json",
					success: function(data) {
						on_get_product_features(data);
					}
				  });
		};

	this.get_dms_status = function (on_get_dms_status)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_dms_status",
					dataType: "json",
					success: function(data) {
						on_get_dms_status(data.dms_running);
					}
				  });
		};

	this.rescan_media = function ()
		{
			$.ajax({
					type: "post",
					url: "/api/v1/rescan_media",
					dataType: "json"
				  });
		};

	this.restart = function ()
		{
			$.ajax({
					type: "post",
					url: "/api/v1/restart",
					dataType: "json"
				  });
		};

	this.get_move_folder = function (on_get_move_folder)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_move_folder",
					dataType: "json",
					success: function(data) {
						on_get_move_folder(data.move_folder);
					}
				  });
		};

	this.set_move_folder = function (move_folder, auto_create, on_set_move_folder)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/set_move_folder",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'move_folder':move_folder,'auto_create':auto_create}),
					dataType: "json",
					success: function(data) {
						on_set_move_folder(data.result);
					}
				  });
		};

	this.get_shared_folder_list = function (on_get_shared_folders)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/get_shared_folder_list",
					dataType: "json",
					success: function(data) {
						on_get_shared_folders(data.shared_folder_list);
					}
				  });
		};

	this.set_shared_folder_list = function (shared_folder_list, on_shared_folder_list)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/set_shared_folder_list",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'shared_folder_list':shared_folder_list}),
					dataType: "json",
					success: function(data) {
						on_shared_folder_list(data.result);
					}
				  });
		};

	this.add_shared_folder = function (shared_folder)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/add_shared_folder",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'shared_folder':shared_folder}),
					dataType: "json"			
				  });
		};

	this.remove_shared_folder = function (shared_folder)
		{
			$.ajax({
					type: "post",
					url: "/api/v1/remove_shared_folder",
					contentType: 'application/json; charset=utf-8',
					data: JSON.stringify({'shared_folder':shared_folder}),
					dataType: "json"			
				  });
		};
}

function MediaItem(udn, id, title, is_container) {
	this.udn = udn;
	this.id = id;
	this.title = title;
	this.is_container = is_container;
}

var dms = new MediaServer();

if (!Array.prototype.indexOf) {
	Array.prototype.indexOf = function(val) {              
		for (var i = 0; i < this.length; i++) {  
			if (this[i] == val) return i;  
		}  
		return -1;  
	};
}

if (!Array.prototype.remove) {
	Array.prototype.remove = function(val) {  
		var index = this.indexOf(val);  
		if (index > -1) {  
			this.splice(index, 1);  
		}  
	}; 
}

if (!Array.prototype.forEachA) {  
    Array.prototype.forEachA = function(callback, thisArg) {  
        var T, k;  
        if (this == null) {  
            throw new TypeError(" this is null or not defined");  
        }  
        var O = Object(this);  
        var len = O.length >>> 0; // Hack to convert O.length to a UInt32  
        if ({}.toString.call(callback) != "[object Function]") {  
            throw new TypeError(callback + " is not a function");  
        }  
        if (thisArg) {  
            T = thisArg;  
        }  
        k = 0;  
        while (k < len) {  
            var kValue;  
            if (k in O) {  
                kValue = O[k];  
                if(!callback.call(T, kValue, k, O)) break;
            }  
            k++;  
        }  
    };  
}  

if (!Array.prototype.forEach) {
	Array.prototype.forEach = Array.prototype.forEachA;
}

if(!String.prototype.format) {
	String.prototype.format = function(args) {
    var result = this;
    if (arguments.length > 0) {    
        if (arguments.length == 1 && typeof (args) == "object") {
            for (var key in args) {
                if(args[key]!=undefined){
                    var reg = new RegExp("({" + key + "})", "g");
                    result = result.replace(reg, args[key]);
                }
            }
        }
        else {
            for (var i = 0; i < arguments.length; i++) {
                if (arguments[i] != undefined) {
                    //var reg = new RegExp("({[" + i + "]})", "g");//这个在索引大于9时会有问题，谢谢何以笙箫的指出
　　　　　　　　　　　　var reg= new RegExp("({)" + i + "(})", "g");
                    result = result.replace(reg, arguments[i]);
                }
            }
        }
    }
    return result;
}
}

if(!Date.prototype.format) {
	Date.prototype.format = function(fmt) {
	  var o = {
		"M+" : this.getMonth()+1,
		"d+" : this.getDate(),
		"h+" : this.getHours(),
		"m+" : this.getMinutes(),
		"s+" : this.getSeconds(), 
		"q+" : Math.floor((this.getMonth()+3)/3),
		"S"  : this.getMilliseconds()
	  };
	  if(/(y+)/.test(fmt))
		fmt=fmt.replace(RegExp.$1, (this.getFullYear()+"").substr(4 - RegExp.$1.length));
	  for(var k in o)
		if(new RegExp("("+ k +")").test(fmt))   
	  fmt = fmt.replace(RegExp.$1, (RegExp.$1.length==1) ? (o[k]) : (("00"+ o[k]).substr((""+ o[k]).length)));   
	  return fmt;
	}
}

function blockUI(selector, show) {
	if(show)
		$(selector).block({message:"<div class='spinner'><div class='bounce1'></div><div class='bounce2'></div><div class='bounce3'></div></div>"});	
	else
		$(selector).unblock();
}

var ARIB_WORD_ARRAY =
{
	'\ue0f8' : '[HV]',
	'\ue0f9' : '[SD]',
	'\ue0fa' : '[Ｐ]',
	'\ue0fb' : '[Ｗ]',
	'\ue0fc' : '[MV]',
	'\ue0fd' : '[手]',
	'\ue0fe' : '[字]',
	'\ue0ff' : '[双]',
	'\ue180' : '[デ]',
	'\ue181' : '[Ｓ]',
	'\ue182' : '[二]',
	'\ue183' : '[多]',
	'\ue184' : '[解]',
	'\ue185' : '[SS]',
	'\ue186' : '[Ｂ]',
	'\ue187' : '[Ｎ]',
	'\ue188' : '■',
	'\ue189' : '●',
	'\ue18a' : '[天]',
	'\ue18b' : '[交]',
	'\ue18c' : '[映]',
	'\ue18d' : '[無]',
	'\ue18e' : '[料]',
	'\ue18f' : '[制限]',
	'\ue190' : '[前]',
	'\ue191' : '[後]',
	'\ue192' : '[再]',
	'\ue193' : '[新]',
	'\ue194' : '[初]',
	'\ue195' : '[終]',
	'\ue196' : '[生]',
	'\ue197' : '[販]',
	'\ue198' : '[声]',
	'\ue199' : '[吹]',
	'\ue19a' : '[PPV]',
	'\ue19b' : '(秘)',
	'\ue19c' : 'ほか',
	'\u301c' : '～'
};

function arib_word_convert(str) {
	if (!str) {
		return str;
	}
    var ret = "";
	for(var i = 0; i < str.length; i++) {
		var c = str.charAt(i);
		if (ARIB_WORD_ARRAY[c] != undefined) {
			ret += ARIB_WORD_ARRAY[c];
		} else {
			ret += c;
		}
	}

	return ret;	
}

function mediaitem_arib_word_convert(item) {
	item.title = arib_word_convert(item.title);
}

function mediaitemlist_arib_word_convert(itemlist) {	
	if($.isArray(itemlist)) {
		for(var i = 0; i < itemlist.length; i++) {
			mediaitem_arib_word_convert(itemlist[i]);
		}
	} else {
		mediaitem_arib_word_convert(itemlist);
	}
}

var GENRE_ARRAY = 
{
	'ドラマ' : 'images/genre_drama.png',
	'映画' : 'images/genre_theater.png',
	'スポーツ' : 'images/genre_sport.png',
	'音楽' : 'images/genre_music.png',
	'バラエティ' : 'images/genre_variety.png',
	'アニメ／特撮' : 'images/genre_animation.png',
	'趣味／教育' : 'images/genre_education.png',
	'ニュース／報道' : 'images/genre_news.png',
	'情報／ワイドショー' : 'images/genre_information.png',
	'ドキュメンタリー／教養' : 'images/genre_documentary.png',
	'劇場／公演' : 'images/genre_theater.png',
	'福祉' : 'images/genre_like.png',
	'その他' : 'images/genre_other.png'
};

function get_mediaitem_thumbnail(item) {
	var thumbnail = GENRE_ARRAY[item.genre];
	if (thumbnail == undefined)
		return "images/genre_other.png";
	else
		return thumbnail;
}

function zero_padding(value, length) {
	var zeros = '';
	value = String(value);
	for (var i = 0; i < (length - value.length); i++) {
		zeros += '0';
	}
	return zeros + value;
}

function get_mediaitem_time_info(item) {
	try {
		var duration = item.duration;
		if (duration > 0) {
			if (item.date != undefined) {
				var date_start = new Date(Date.parse(item.date));
				var date_end = new Date();
				date_end.setTime(date_start.getTime() + duration);
				var time_info =
					  date_start.format("yyyy/MM/dd hh:mm:ss")
					+ date_end.format("-hh:mm:ss");
				return time_info;
			} else {
				duration = Math.floor(duration / 1000);
				var hour = Math.floor((duration / 3600));
				var min  = Math.floor((duration - hour * 3600) / 60);
				var sec = duration - hour * 3600 - min * 60;
				var time_info = "";
				if (hour < 10)
					time_info += "0";
				time_info += hour;
				time_info += ":";
				if (min < 10)
					time_info += "0";
				time_info += min;
				time_info += ":";
				if (sec < 10)
					time_info += "0";
				time_info += sec;				
				return time_info;
			}
		}
	} catch(err) {
		console.log(err);
	}	
	return "";	
}

function mediaitem_compare_function(a, b) {
	if (a.is_container && !b.is_container) {
		return -1;
	} else if (!a.is_container && b.is_container) {
		return 1;
	} else {
		if (a.title < b.title)
			return -1;
		else if (a.title > b.title) {
			return 1;
		} else {
			return 0;
		}
	}
}

function mediaitem_is_downloadable(item) {
	return item.is_downloadable && item.title != "******";
}

function mediaitem_is_selectable(item) {
	return item.is_container || item.title != "******";	
}

function get_path_name(path) {
	var index = path.lastIndexOf("/");
	return path.substr(index+1);
}

function get_path_parent(path) {
	var index = path.lastIndexOf("/");
	return path.substr(0, index);
}

function jquery_escape(id) {
	return id.replace( /(:|\.|\[|\]|,)/g, "\\$1" );
}

function update_dynamic_ui() {
	dms.get_product_features(function(data) {
		if (data.dms_enable_ras) {
			$("#link_paired_devices").show();
		} else {
			$("#link_paired_devices").hide();
		}
		$("#online_help_link").attr("href", data.online_help_link);
	});

	var includes = $(".include");
    jQuery.each(includes, function() {
		var file = $(this).data('include');
		$(this).load(file);
    });
}

jQuery(document).ready(function($) {
	update_dynamic_ui();
});



